#!/bin/ksh
#> This script will install the Prospect patch
#> Usage: install.sh {-help|-install|-uninstall}
#
typeset    sql
typeset    pth
typeset    opt
typeset -i OK
typeset    MW

function do_sql
{
echo "Executing sqlplus -> $sql";
sqlplus -s /NOLOG << eof
connect $DBUSER/$DBPASSWD@$DBSID
@${pth}/${sql}
exit;
;;
eof
}

function do_wrap
{
    x=`basename $sql .sql`;
    y=`basename $sql`;
    if [ ${x}.sql = ${y} ]
    then
        $ORACLE_HOME/bin/wrap iname=$sql oname=./tmp.sql
        chmod +w ${sql}
	mv ./tmp.sql ${sql}
        chmod -w ${sql}
    fi
}

function do_mw_stop
{
    typeset MW
    MW=`ps -ef|grep wmpm|grep service|grep ${PM_PORT}`
    if [[ "${MW}t" != "t" ]]; then
        echo ""
        echo "Shutting down the middleware..."
        ps-mgr stop all; ps-mgr halt
        MW=`ps -ef|grep wmpm|grep service|grep ${PM_PORT}`
        if [[ "${MW}t" != "t" ]]; then
            echo ""
            echo "Unable to shut down the middleware."
            echo "Please, perform the shutting down before proceeding with installation."
            echo ""
            exit;
        fi
        echo "Middleware correctly stopped"
        echo ""
    fi
}

function do_mw_start
{
    typeset MW
    MW=`ps -ef|grep wmpm|grep service|grep ${PM_PORT}`
    if [[ "${MW}t" = "t" ]]; then
        echo ""
        echo "Starting up the middleware..."
        ps-mgr init
        MW=`ps -ef|grep wmpm|grep service|grep ${PM_PORT}`
        if [[ "${MW}t" = "t" ]]; then
            echo ""
            echo "Unable to re-start the middleware."
            echo "Try to perform a manual startup."
            echo ""
            exit;
        fi
        echo "Middleware correctly started"
        echo ""
    fi
}

########################################################
# set paths and files
COMPONENT_TYPE="Core"
REVISION="8.0.4"
PATCH="7"
FIX="P${PATCH}"
THISBLD="04"
BLD="${REVISION}.${PATCH}"
RELEASE="${COMPONENT_TYPE}${REVISION}_${FIX}"
SourceDir="${FLEXPM_HOME}/patches/${RELEASE}"
SourceDir1="${FLEXPM_HOME}/patches/${RELEASE}/install"
BackupDir1="${FLEXPM_HOME}/patches/${RELEASE}/backup/install"
INSTALLDIR=`pwd`
########################################################
ExecuteInstallBefore='
dew_drops.sql
db/Oracle8i/bh/busyhour_taskDrop.sql
schema_patch.sql
'
ExecuteInstallAfter='
compile-schema.sql
patch.sql
upgrade_summations.sql
'
ExecuteUninstallBefore='
dew_drops.sql
db/Oracle8i/bh/busyhour_taskDrop.sql
schema_patch.sql
'
ExecuteUninstallAfter='
compile-schema.sql
downgrade_summations.sql
'
DoNotCompile='
db/Oracle8i/bh/busyhour_taskDrop.sql
db/Oracle8i/part_maint/createTables.sql
db/Oracle8i/part_maint/part_statDrop.sql
db/Oracle8i/part_maint/part_taskDrop.sql
db/Oracle8i/summarize/summ_taskDrop.sql
'
InstallFileset1='
db/Oracle8i/bh/BH_Driver_tmp.sql
db/Oracle8i/bh/busyhour_taskDrop.sql
db/Oracle8i/bh/busyhour_task.sql
db/Oracle8i/part_maint/createTables.sql
db/Oracle8i/part_maint/createTriggers.sql
db/Oracle8i/part_maint/part_taskDrop.sql
db/Oracle8i/part_maint/part_task.sql
db/Oracle8i/part_maint/part_statDrop.sql
db/Oracle8i/part_maint/part_stat.sql
db/Oracle8i/summarize/summ_taskDrop.sql
db/Oracle8i/summarize/summ_task.sql
db/Oracle8i/scheduler/sched/schedule_maint
db/Oracle8i/scheduler/except/except.sql
db/Oracle8i/bh/BH_UpdatePack.sql
db/Oracle8i/bh/BH_Update.sql
db/Oracle8i/part_maint/part_maintPack.sql
db/Oracle8i/part_maint/part_maint.sql
db/Oracle8i/cfgtool/create_schema_objects.sql
db/Oracle8i/softalarm/sacheck/schedcheck.sql
db/Oracle8i/summarize/CreateSumTab.sql
db/Oracle8i/summarize/summarizePack.sql
db/Oracle8i/summarize/summarize.sql
db/Oracle8i/wmlib/wm_sysval_pkg.sql
db/Oracle8i/wmlib/wm_sysval_pkgb.sql
db/Oracle8i/cfgtool/ConfigToolParser11.sql
db/Oracle8i/utility/autocleanPack.sql
db/Oracle8i/utility/autoclean.sql
db/Oracle8i/utility/sched_immediate_pkg.sql
db/Oracle8i/utility/sched_immediate_pkgb.sql
db/Oracle8i/dictionary/wmn_pkgb.sql
db/Oracle8i/rg/rg_common.sql
sw/oracle/compile-schema.sql
peik/loaders/scripts/file_decode.pl
pm/adm/import_docs.sh
pm/adm/exp_udc_convert.pl
pm/adm/part_mask.sh
pm/adm/wmpm-init
pm/adm/wmpm.sh
pm/bin/loader9i
pm/bin/loader10g
pm/bin/commander9i
pm/bin/commander10g
pm/bin/doproc
pm/run/bhd.sh
pm/run/bhrgp.sh
pm/run/download
pm/run/ps-mgr
pm/run/rgp.sh
pm/run/udr.sh
pm/bin/rgp
pm/lib/wm/libdataDictionary.so
pm/lib/wm/libexpressions.so
pm/lib/wm/libReportEngine.so
pm/lib/wm/libWmGateway.so
'
### give help
  ToolName=`basename $0`
  opt=$1
  [[ ${opt}x = "x" ]] && opt="-install"
  case $opt in
  -help|-usage)
    echo "${ToolName} usage information -- " 1>&2
    grep "^#> " $0 1>&2
    grep "^#>  \"" $0 1>&2
    exit 2
    ;;
  esac
# Check env settings
  if [ $# -ne 1 ]; then
    echo "Usage $0 {-help | -install | -uninstall}"
    exit;
  fi
  echo "Checking environment....."
  if ! [[ -d ${FLEXPM_HOME} ]]; then
    echo "Env variable FLEXPM_HOME must be set to appropriate dir before installation."
    exit;
  fi
  if [[ -z ${DB_CONNECT} ]]; then
     echo "Env variable DB_CONNECT must be set to appropriate database connection values."
     exit;
  fi

  case $opt in
	  -install| -uninstall)
  		if [[ ${INSTALLDIR} != ${SourceDir} ]] ; then
     		  echo "This script should be run from ${SourceDir}"
     		  exit;
  		fi
	  ;;
  esac

  if [[ ! -a ./install/${RELEASE}.build ]]
  then
    echo "./install/${RELEASE}.build file not found"
    exit;
  fi
  if [[ $BLD != `grep $BLD ./install/${RELEASE}.build | awk '{ print $4 }'` ]]
  then
   echo "./install/${RELEASE}.build Version does not match $BLD"
   cat ./install/${RELEASE}.build
   exit;
  else
     if [[ $THISBLD != \
       `grep $THISBLD ./install/${RELEASE}.build | awk '{ print $5 }'` ]]
     then
      echo "./install/${RELEASE}.build Build does not match $THISBLD"
      cat ./install/${RELEASE}.build 
      exit;
     fi
  fi


# Source and destination directories

  case $opt in
  -install)
    echo "Installing patch..."

    if [ -a ${BackupDir1}/pm/bin/doproc ]
    then
        echo ""
        echo "Patch $RELEASE may already have been applied. Please verify."
        echo ""
        exit
    fi
    do_mw_stop;
    echo "Registration of ${RELEASE} started."
    chmod +x $FLEXPM_HOME/pm/adm/register 
    register start ${SourceDir1}/${RELEASE}.build
    [[ $? -ne 0 ]] && exit 1;
    echo "Files to be updated/added ..."
    pth=$SourceDir1
    for sql in ${ExecuteInstallBefore}
    do
       x=`basename $sql .sql`;
       y=`basename $sql`;
       [[ ${x}.sql = ${y} ]] && do_sql;
    done
    for f in ${InstallFileset1}
    do
	d=`dirname ${f}`
	[[ ! -d ${FLEXPM_HOME}/${d} ]] && mkdir -p ${FLEXPM_HOME}/${d} 
	chmod -R u+wx ${FLEXPM_HOME}/${d}
        echo "${f}"
        if [[ -a ${FLEXPM_HOME}/${f} ]]; then
        # permissions
           chmod u+w ${FLEXPM_HOME}/${f}
        # backup
           cp -pf ${FLEXPM_HOME}/${f} ${BackupDir1}/${f}
        fi
        # install files
	cp -pf ${SourceDir1}/${f} ${FLEXPM_HOME}/${f}
		
	# permissions
	chmod u-w ${FLEXPM_HOME}/${f}
	chmod u+x ${FLEXPM_HOME}/${f}
	#execute sqlplus for those file ending in .sql 
	OK=1
	for DoNot in ${DoNotCompile}
	do
	   if [[ ${f} = $DoNot ]]; then
		OK=0
	   	#but not for sql in the DoNotCompile list
        	echo ""
		echo "Note: ${DoNot} was not compiled on purpose"
        	echo ""
	   fi
	done
	if [[ $OK -eq 1 ]] ; then
	   sql=$f
	   x=`basename $sql .sql`;
	   y=`basename $sql`;
	   [[ ${x}.sql = ${y} ]] && do_sql;
	fi
    done 
    for sql in ${ExecuteInstallAfter}
    do
	   x=`basename $sql .sql`;
	   y=`basename $sql`;
	   [[ ${x}.sql = ${y} ]] && do_sql;
    done
    ${SourceDir1}/recompile_view.sh
    register complete ${SourceDir1}/${RELEASE}.build
    do_mw_start;
    cd ${SourceDir}
    echo "Installation completed successfully."
    ;;

  -uninstall)
    echo "Uninstalling patch..."
    if [ ! -a ${BackupDir1}/pm/bin/doproc ]
    then
        echo ""
        echo "Patch $RELEASE may not have been applied yet."
        echo "Nothing to uninstall. Please verify."
        echo ""
        exit
    fi
    echo "Restoring original files..."
    do_mw_stop;
    pth=$SourceDir1
    for sql in ${ExecuteUninstallBefore}
    do
       x=`basename $sql .sql`;
       y=`basename $sql`;
       [[ ${x}.sql = ${y} ]] && do_sql;
    done
    pth=$BackupDir1
#Re-install previous objects
    for f in ${InstallFileset1}
    do
        echo "${f}"
        # permissions
        chmod u+w ${FLEXPM_HOME}/${f}

        if [[ -a ${BackupDir1}/${f} ]]; then
           # restore files from backup
           cp -fp ${BackupDir1}/${f} ${FLEXPM_HOME}/${f}
	   OK=1
        else
	   OK=0
           rm -f ${FLEXPM_HOME}/${f}
        fi
        if [[ -a ${FLEXPM_HOME}/${f} ]]; then
	   if [[ $OK -eq 1 ]] ; then
             # permissions
             chmod u-w ${FLEXPM_HOME}/${f}
             chmod u+x ${FLEXPM_HOME}/${f}
           fi
        fi
	for DoNot in ${DoNotCompile}
	do
	   if [[ $f = $DoNot ]]; then
		OK=0
		echo "${DoNot} was not compiled on purpose"
	   fi
	done
	if [[ $OK -eq 1 ]] ; then
	   sql=$f
           x=`basename $sql .sql`;
	   y=`basename $sql`;
	   [[ ${x}.sql = ${y} ]] && do_sql;
	fi
        # remove files from backup
        rm -rf ${BackupDir1}/${f}
    done
    pth=$SourceDir1
    for sql in ${ExecuteUninstallAfter}
    do
       x=`basename $sql .sql`; 
       y=`basename $sql`; 
       [[ ${x}.sql = ${y} ]] && do_sql;
    done
    pth=$BackupDir1
    rm -f ${BackupDir1}/bin.tar
    rm -f ${BackupDir1}/lib.tar
    ${SourceDir1}/recompile_view.sh
    #Remove the show_installed for this patch
    sq1="select component_version_id from component_version \
     where revision = '${REVISION}.${PATCH}' \
       and component_id = (select component_id from component \
       where component_type = upper('${COMPONENT_TYPE}')) \
      and build_number = $THISBLD"
    sq2="delete component_history where component_version_id = ( $sq1 )"
    sq3="delete component_installed where component_version_id = ( $sq1 )"
    sq4="delete component_version where component_version_id = ( $sq1 )"
sqlplus -s /NOLOG << eof
connect $DBUSER/$DBPASSWD@$DBSID
${sq2};
${sq3};
${sq4};
exit;
eof
    do_mw_start;
    echo "Un-Installation completed successfully."
    cd ${SourceDir}
    ;;
  
   -diff | -get | -wrap )
      for f in ${InstallFileset1} 
      do 
        case $opt in 
	-wrap)
		sql=${INSTALLDIR}/install/${f}
		do_wrap;
	   	;;
         -diff | -get )
            CP=`diff -bitw ./install/${f} ${FLEXPM_HOME}/${f}`
	    x="$?"
	    case $x in
	     0 )
	      case $opt in
               -diff)
		echo ">OK ./install/${f}<" 
	        ;;
              esac
	      ;;
	     1 )
	      case $opt in
               -diff)
		echo "different ${f}" 
      		;;
               -get)
		chmod +w ./install/${f}
		cp ${FLEXPM_HOME}/${f} ./install/${f}
		chmod -w ./install/${f}
	   	;;
              esac
	      ;;
	     2 )
	      case $opt in
               -diff)
		echo "missing ${f}" 
      		;;
               -get)
		cp ${FLEXPM_HOME}/${f} ./install/${f}
		chmod -w ./install/${f}
	   	;;
               esac
	      ;;
        esac
        ;;
       esac
      done 
      ;;
  esac

